using System;
using System.Web.Mvc;
using System.Web.Routing;
using MvcGallery3.Extensions.Localization;

namespace MvcGallery3.Extensions.ViewEngines
{
    public class LocalizableRazorViewEngine : RazorViewEngine 
    {
        protected override IView CreatePartialView(ControllerContext controllerContext, String partialPath)
        {
            var localizedPath = partialPath;
            if (!String.IsNullOrEmpty(partialPath))
                localizedPath = GetLocalizedViewName(controllerContext.RequestContext, partialPath);
            return base.CreatePartialView(controllerContext, localizedPath);
        }

        protected override IView CreateView(ControllerContext controllerContext, String viewPath, String masterPath)
        {
            var localizedView = viewPath;
            var localizedMaster = masterPath;

            if (!String.IsNullOrEmpty(viewPath))
                localizedView = GetLocalizedViewName(controllerContext.RequestContext, viewPath);
            if (!String.IsNullOrEmpty(masterPath))
                localizedMaster = GetLocalizedViewName(controllerContext.RequestContext, masterPath);
            return base.CreateView(controllerContext, localizedView, localizedMaster);
        }

        private static String GetLocalizedViewName(RequestContext context, String view)
        {
            var urlHelper = new UrlHelper(context);
            return UrlExtensions.GetLocalizedUrl(urlHelper, view);
        }
    }
}