﻿<%@ Page Language="C#" MasterPageFile="~/Views/Shared/Site.Master" Inherits="System.Web.Mvc.ViewPage<HomeIndexViewModel>" %>
<%@ Import Namespace="Northwind.Model" %>
<%@ Import Namespace="ServerControls.Models" %>


<script runat="server">
    protected void Page_Load(object sender, EventArgs e)
    {
        var model = ViewData.Model;

        // Do not rebind the drop-down list. The drop-down is already
        // filled thanks to the viewstate. Binding again would lose you
        // the selection.
        if (!IsPostBack)       // Always false in ASP.NET MVC 2+
            BindCustomers(model);
        
        BindOrders(model);
    }

    // Fully supported by ASP.NET MVC 2
    protected void gridOrders_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        // Add a tooltip with the company address
        var model = e.Row.DataItem as Order;
        if (model == null)
            return;
        var address = model.Customer.Address;
        e.Row.ToolTip = address;
    }

    // Never hit in ASP.NET MVC2
    protected void gridOrders_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        gridOrders.PageIndex = e.NewPageIndex;
        BindOrders(ViewData.Model);
    }

    private void BindOrders(HomeIndexViewModel model)
    {
        // Manual paging for a grid control
        int index = 0;
        string eventTarget = Request.Form["__EVENTTARGET"];
        string eventArgument = Request["__EVENTARGUMENT"];
        if (String.Equals(eventTarget, gridOrders.UniqueID))
        {
            if (!String.IsNullOrEmpty(eventArgument))
            {
                string[] tokens = eventArgument.Split('$');
                index = Convert.ToInt32(tokens[1]) - 1; // not 0-based
            }
        }

        // Data binding
        if (index > 0)
            gridOrders.PageIndex = index;
        gridOrders.DataSource = model.Orders;
        gridOrders.DataBind();
    }

    private void BindCustomers(HomeIndexViewModel model)
    {
        ddCustomerList.DataSource = model.Customers;
        ddCustomerList.DataBind();
        ddCustomerList.SelectedIndex = SelectedIndexById();
    }
    
    private int SelectedIndexById()
    {
        var index = 0;
        string id = Request[ddCustomerList.UniqueID];
        foreach (Customer c in (IEnumerable)ddCustomerList.DataSource)
        {
            if (c.CustomerID == id)
                return index;
            else
                index++;
        }
        return index;
    }
</script>


<asp:Content ID="indexTitle" ContentPlaceHolderID="TitleContent" runat="server">
   Testing server controls in MVC
</asp:Content>

<asp:Content ID="indexContent" ContentPlaceHolderID="MainContent" runat="server">
    <h2>DataGrid in action</h2>
    <form id="Form1" runat="server">
        <asp:DropDownList runat="server" ID="ddCustomerList" 
            ClientIDMode="Static"
            AutoPostBack="true"
            DataTextField="CompanyName"
            DataValueField="CustomerID" /> 
        <hr />
        <asp:GridView ID="gridOrders" runat="server" 
                AutoGenerateColumns="false" 
                BackColor="White"
                CellPadding="3" 
                CellSpacing="1" 
                GridLines="None"
                rules="rows" 
                frame="hsides"  
                Style="border-top:solid 1px darkgray;border-width:1px;text-align:center;"
                HeaderStyle-BackColor="LightBlue" 
                OnRowDataBound="gridOrders_RowDataBound">
                <Columns>
                    <asp:BoundField DataField="OrderID" HeaderText="ID" />
                    <asp:BoundField DataField="CustomerID" HeaderText="Customer"
                            ItemStyle-Width="100px" />
                    <asp:BoundField DataField="OrderDate" HeaderText="Order issued" 
                            DataFormatString="{0:dd MMM yy}" 
                            ItemStyle-Width="100px" />
                </Columns>
        </asp:GridView>
    </form>    
</asp:Content>
