﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using MvcGallery3.Utils;

namespace Engines.Controllers
{
    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            ViewBag.Message = "Registered view engines:";
            ViewData["DefaultText"] = DateTime.Today.ToShortDateString();
            ViewBag.EngineList = GetRegisteredViewEngines();

            return View();
        }

        public ActionResult About()
        {
            ViewData["Message"] = "Thank you for choosing this book!";
            return View();
        }

        public String Details(String engineList)
        {
            String output = String.Empty;

            var engines = engineList.Split(',');
            foreach(var engine in engines)
            {
                // Grab a reference to the engine
                var e = GetViewEngineReference(engine);
                output += BuildTextForEngine(e);
            }

            return output;
        }

        private static IList<String> GetRegisteredViewEngines()
        {
            return ViewEngines.Engines.Select(engine => engine.ToString()).ToList();
        }

        private static VirtualPathProviderViewEngine GetViewEngineReference(String engineName)
        {
            // Assuming you didn't change the order of view engines!!!

            if (engineName.Contains("RazorViewEngine"))
                return ViewEngines.Engines[1] as VirtualPathProviderViewEngine;

            return ViewEngines.Engines[0] as VirtualPathProviderViewEngine;
        }

        private static String BuildTextForEngine(VirtualPathProviderViewEngine engine)
        {
            var builder = new StringBuilder();
            builder.AppendFormat("<table>");
            builder.AppendFormat("<thead><th rowspan=2>{0}</thead>", engine.ToString());
            builder.AppendFormat("<tr><td>AreaMasterLocationFormats</td><td>{0}</td></tr>", engine.AreaMasterLocationFormats.AsCommaSeparated());
            builder.AppendFormat("<tr><td>AreaPartialViewLocationFormats</td><td>{0}</td></tr>", engine.AreaPartialViewLocationFormats.AsCommaSeparated());
            builder.AppendFormat("<tr><td>AreaViewLocationFormats</td><td>{0}</td></tr>", engine.AreaViewLocationFormats.AsCommaSeparated());
            builder.AppendFormat("<tr><td>MasterLocationFormats</td><td>{0}</td></tr>", engine.MasterLocationFormats.AsCommaSeparated());
            builder.AppendFormat("<tr><td>PartialViewLocationFormats</td><td>{0}</td></tr>", engine.PartialViewLocationFormats.AsCommaSeparated());
            builder.AppendFormat("<tr><td>ViewLocationFormats</td><td>{0}</td></tr>", engine.ViewLocationFormats.AsCommaSeparated());
            builder.AppendFormat("<tr><td>Extensions</td><td>{0}</td></tr>", engine.FileExtensions.AsCommaSeparated());
            builder.AppendFormat("</table>");
            return builder.ToString();
        }
    }
}
