using System;
using System.Web.Mvc;
using System.Web.WebPages;

namespace RazorEngine.App_Code
{
    public static class WebPageExtensions
    {
        public static HelperResult RenderSection(this WebViewPage thePage, 
            String sectionName,
            Func<Object, HelperResult> funcDefaultContent)
        {
            // Use section template if specified in actual content page.
            if (thePage.IsSectionDefined(sectionName))
            {
                return thePage.RenderSection(sectionName);
            }

            // Use section template if specified in _viewstart page through PageData dictionary.
            if (thePage.PageData.ContainsKey(sectionName))
            {
                var templateFunc = thePage.PageData[sectionName] as Func<Object, HelperResult>;
                if (templateFunc != null)
                    return templateFunc(null);
            }

            // Go with default content as passed as an argument here.
            return funcDefaultContent(null);
        }
    }

    public static class MvcIntrinsics
    {
        public static System.Web.Mvc.HtmlHelper Html
        {
            get { return ((System.Web.Mvc.WebViewPage)WebPageContext.Current.Page).Html; }
        }
    }

}