﻿using System;
using System.Diagnostics.Contracts;
using System.Web.Mvc;
using BasicInput.ViewModels.Binding;

namespace BasicInput.Controllers
{
    public class BindingController : Controller
    {
        public ActionResult Repeat(String text, Int32 number)
        {
            Contract.Requires<ArgumentException>(text.Length >0);
            Contract.Requires<ArgumentException>(number > 0 && number <10);

            var model = new RepeatViewModel { Number = number, Text = text };
            return View("repeat", model);
        }
   
        public ActionResult RepeatNull(String text, Int32? number)
        {
            var model = new RepeatViewModel {Number = number.GetValueOrDefault(), Text = text};
            return View("repeat", model);
        }
        
        public ActionResult RepeatDefault(String text, Int32 number=4)
        {
            var model = new RepeatViewModel { Number = number, Text = text };
            return View("repeat", model);
        }

        public ActionResult ShowDate(Boolean shouldShowTime)
        {
            var model = new ShowDateViewModel { ShouldShowTime = shouldShowTime, Today = DateTime.Now };
            return View(model);
        }

        /// <summary>
        /// Index page has a link to this method where NUMBER is set as a route parameter.
        /// That value takes precedence over query string param and default values.
        /// </summary>
        /// <param name="text">Text to repeat</param>
        /// <param name="number">Times to repeat</param>
        /// <returns></returns>
        public ActionResult RepeatWithPrecedence(String text, Int32 number=20)
        {
            var model = new RepeatViewModel { Number = number, Text = text };
            return View("repeat", model);
        }
    }
}
