﻿using System;
using System.Web.Mvc;
using AjaxForm.WorkerServices.Abstractions;
using AjaxForm.WorkerServices.Customers;
using MvcGallery3.Extensions.Filters;
using Northwind.Model;

namespace AjaxForm.Controllers
{
    public class CustomerController : Controller
    {
        private readonly ICustomerWorkerServices _service;

        public CustomerController() : this(new CustomerWorkerServices())
        {
        }
        public CustomerController(ICustomerWorkerServices service)
        {
            _service = service;
        }

        [AjaxOnly]
        public ActionResult Edit([Bind(Prefix = "customerList")] String customerId)
        {
            var model = _service.EditCustomer(customerId);

            // Add a custom response header to easily pass extra information to the 
            // client Ajax engine--alternative would be packing everything into JSON and NOT 
            // using the MVC Ajax infrastructure. Name of the header is arbitrary.
            if (Request.IsAjaxRequest())
                HttpContext.Response.AddHeader("Content-Title", model.Customer.CompanyName);
            return PartialView("_customerEditor", model);
        }

        public ActionResult Update(Customer customer)
        {
            var model = _service.TryUpdateCustomer(ControllerContext, customer);
            return PartialView("_customerEditor", model);
        }
    }
}
