using System;
using System.Web.Mvc;
using AjaxForm.ViewModels.Customers;
using AjaxForm.WorkerServices.Abstractions;
using Northwind.DAL.Repositories;
using Northwind.DAL.Repositories.Abstractions;
using Northwind.Model;

namespace AjaxForm.WorkerServices.Customers
{
    public class CustomerWorkerServices : ICustomerWorkerServices
    {
        private readonly ICustomerRepository _repository;

        public CustomerWorkerServices() : this(new CustomerRepository())
        {
        }
        public CustomerWorkerServices(ICustomerRepository repository)
        {
            _repository = repository;
        }

        public ListCustomersViewModel FindAllCustomers()
        {
            var model = new ListCustomersViewModel 
                            {
                                Title = "Customers",
                                Customers = _repository.FindAll()
                            };
            return model;
        }

        public EditCustomerViewModel EditCustomer(String id)
        {
            var model = new EditCustomerViewModel
                            {
                                Title = "Edit customer",
                                Customer = _repository.FindById(id) 
                            };
            return model;
        }

        public EditCustomerViewModel TryUpdateCustomer(ControllerContext context, Customer customer)
        {
            var success = false;
            var isValid = Validate(context, customer);
            var model = EditCustomer(customer.CustomerID);
            if (isValid)
                success = Update(context, customer);

            model.OutputMessage = "Update failed. Check your input data!";
            if (success)
                model.OutputMessage = "Successfully updated.";

            return model;
        }
        
        private static Boolean Validate(ControllerContext context, Customer customer)
        {
            var result = true;
            if (String.IsNullOrEmpty(customer.Country) || !customer.Country.Equals("USA"))
            {
                var modelState = context.Controller.ViewData.ModelState;
                modelState.AddModelError("Country",
                    "Invalid country. Can only be 'USA'.");

                // Save model-state to TempData
                //context.Controller.TempData["ModelState"] = modelState;
                result = false;
            }

            return result;
        }

        private Boolean Update(ControllerContext context, Customer customer)
        {
            return _repository.Update(customer);
        }
    }
}