using System;
using System.Web.Mvc;
using Northwind.DAL.Repositories;
using Northwind.DAL.Repositories.Abstractions;
using Northwind.Model;
using SEP.ViewModels.Customers;
using SEP.WorkerServices.Abstractions;

namespace SEP.WorkerServices.Customers
{
    public class CustomerWorkerServices : ICustomerWorkerServices
    {
        private readonly ICustomerRepository _repository;

        public CustomerWorkerServices() : this(new CustomerRepository())
        {
        }
        public CustomerWorkerServices(ICustomerRepository repository)
        {
            _repository = repository;
        }

        public EditCustomerViewModelBase FindAllCustomers()
        {
            var model = new EditCustomerViewModelBase {Title="Find all", Customers = _repository.FindAll()};
            return model;
        }

        public EditCustomerViewModel EditCustomer(String id)
        {
            var model = new EditCustomerViewModel
                            {
                                Title = "Edit customer",
                                Customer = _repository.FindById(id),
                                Customers = _repository.FindAll()
                            };
            return model;
        }

        public EditCustomerViewModel TryUpdateCustomer(ControllerContext context, Customer customer)
        {
            if (Validate(context, customer))
                Update(customer);

            return EditCustomer(customer.CustomerID);
        }


        private static Boolean Validate(ControllerContext context, Customer customer)
        {
            // Check also against nullness (the model binder leaves it NULL if not specified)
            var result = true;
            if (String.IsNullOrEmpty(customer.Country) || !customer.Country.Equals("USA"))
            {
                context.Controller.ViewData.ModelState.AddModelError("Country",
                    "Invalid country. ");
                result = false;
            }

            return result;
        }

        private static void Update(Customer customer)
        {
            // Perform physical update here using the repository services
            // _repository.Update(customer);
        }
    }
}