using DotNetOpenAuth.Messaging;
using DotNetOpenAuth.OAuth;
using DotNetOpenAuth.OAuth.ChannelElements;

namespace TwitterAuth.Framework
{
    public static class FacebookConsumer
    {
        /// <summary>
        /// The description of FB's OAuth protocol URIs for use with actually reading/writing
        /// a user's private FB data.
        /// </summary>
        public static readonly ServiceProviderDescription ServiceDescription = new ServiceProviderDescription
                                                                                   {
                                                                                       RequestTokenEndpoint =
                                                                                           new MessageReceivingEndpoint(
                                                                                           "http://twitter.com/oauth/request_token",
                                                                                           HttpDeliveryMethods. GetRequest |
                                                                                           HttpDeliveryMethods.AuthorizationHeaderRequest),
                                                                                       UserAuthorizationEndpoint =
                                                                                           new MessageReceivingEndpoint(
                                                                                           "http://twitter.com/oauth/authorize",
                                                                                           HttpDeliveryMethods.GetRequest |
                                                                                           HttpDeliveryMethods.AuthorizationHeaderRequest),
                                                                                       AccessTokenEndpoint =
                                                                                           new MessageReceivingEndpoint(
                                                                                           "http://twitter.com/oauth/access_token",
                                                                                           HttpDeliveryMethods.GetRequest |
                                                                                           HttpDeliveryMethods.AuthorizationHeaderRequest),
                                                                                       TamperProtectionElements =
                                                                                           new ITamperProtectionChannelBindingElement[]
                                                                                               {
                                                                                                   new HmacSha1SigningBindingElement()
                                                                                               },
                                                                                   };
    }
}