using System;
using DotNetOpenAuth.OAuth.Messages;

namespace TwitterAuth.Framework
{
    /// <summary>
    /// This class is expected to maintain a persistent store tracking user name and access token. The reason is that you retrieve
    /// access token for a given user and use that to issue an authorized call to the service provider (i.e., Twitter). You don't 
    /// need this for simple authentication purposes.
    /// </summary>
    public class PersistentTokenManager : ConfigTokenManagerBase
    {
        // Just use a persistent store instead of an in-memory dictionary like TokensAndSecrets.

        protected override void StoreAccessTokenCore(String consumerKey, String requestToken, String accessToken, String accessTokenSecret)
        {
            // Invoked past authorization, this method exchanges a request token (now authenticated) with an access token.
            TokensAndSecrets.Remove(requestToken);
            TokensAndSecrets[accessToken] = accessTokenSecret;
        }

        protected override String GetTokenSecretCore(String token)
        {
            // Invoked past authorization, this method gets the secret given a request or access token.
            return TokensAndSecrets[token];
        }

        protected override void StoreNewRequestTokenCore(UnauthorizedTokenRequest request, ITokenSecretContainingMessage response)
        {
            // Invoked before going to Twitter to store a newly generated request token for later use.
            TokensAndSecrets[response.Token] = response.TokenSecret;
        }
    }
}