﻿using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using MvcGallery3.Extensions.Factory;

namespace IoCFatFree
{
    public class MvcApplication : HttpApplication
    {
        /// <summary>
        /// Register application-specific global filters.
        /// </summary>
        /// <param name="filters">Built-in filter collection to extend</param>
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        /// <summary>
        /// Register application-specific routes.
        /// </summary>
        /// <param name="routes">Built-in route collection to extend</param>
        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Parameter defaults
            );

        }

        /// <summary>
        /// Register a custom controller factory to use with parameterized ctors
        /// </summary>
        public static void RegisterControllerFactory()
        {
            var factory = new UnityControllerFactory();
            ControllerBuilder.Current.SetControllerFactory(factory);
        }


        /// <summary>
        /// Handler for the application startup point.
        /// </summary>
        protected void Application_Start()
        {
            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);
            RegisterControllerFactory();
        }
    }
}