﻿using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using MvcGallery3.Extensions.Filters;

namespace Misc
{
    public class MvcApplication : HttpApplication
    {
        /// <summary>
        /// Register application-specific global filters.
        /// </summary>
        /// <param name="filters">Built-in collection of global filters to extend</param>
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }

        /// <summary>
        /// Register application-specific filter providers.
        /// </summary>
        /// <param name="providers">Built-in collection of filter providers to extend</param>
        public static void RegisterFilterProviders(FilterProviderCollection providers)
        {
            providers.Add(new DynamicLoadingFilterProvider());
        }

        /// <summary>
        /// Register application-specific routes.
        /// </summary>
        /// <param name="routes">Built-in route collection to extend</param>
        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new { controller = "Home", action = "Index", id = UrlParameter.Optional } // Parameter defaults
            );

        }

        /// <summary>
        /// Handler for the application startup point.
        /// </summary>
        protected void Application_Start()
        {
            RegisterFilterProviders(FilterProviders.Providers);
            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);
        }
    }
}