using System;
using System.Collections.Generic;
using System.Linq;

namespace AgenderCore.Framework
{
    public class ConferenceInfo
    {
        public ConferenceInfo()
        {
            Days = new List<Day>();
            Speakers = new List<Speaker>();
            Sponsors = new List<Sponsor>();
        }

        public String Conference { get; set; }
        public String Location { get; set; }
        public String Description { get; set; }
        public String Date { get; set; }

        public IList<Day> Days { get; set; }
        public IList<Speaker> Speakers { get; set; }
        public IList<Sponsor> Sponsors { get; set; }

        public IList<Session> GetSessionsForSpeaker(String id)
        {
            var sessions = GetAllSessions();
            return (from s in sessions
                    where String.Equals(s.Id, id, StringComparison.InvariantCultureIgnoreCase)
                    select s).ToList();
        }

        public Session GetSessionById(String id)
        {
            var sessions = GetAllSessions();
            return (from s in sessions
                    where String.Equals(s.Id, id, StringComparison.InvariantCultureIgnoreCase)
                    select s).FirstOrDefault();
        }

        public IList<Session> GetAllSessions()
        {
            var sessions = new List<Session>();

            foreach (var day in Days)
            {
                foreach (var slot in day.TimeSlots)
                {
                    var speakerSessions =
                        (from s in slot.Sessions select s).ToList();
                    sessions.AddRange(speakerSessions);
                }
            }

            return sessions;
        }

        public Speaker GetSpeakerById(String id)
        {
            return
                (from s in Speakers where String.Equals(id, s.Id, StringComparison.InvariantCultureIgnoreCase) select s)
                    .FirstOrDefault();
        }

        public Speaker GetSpeakerByName(String name)
        {
            return
                (from s in Speakers where String.Equals(name, s.Name, StringComparison.InvariantCultureIgnoreCase) select s)
                    .FirstOrDefault();
        }
    }
}