using System;
using System.Linq;
using System.Xml.Linq;

namespace AgenderCore.Framework
{
    public class ContentReader
    {
        private static ConferenceInfo _conferenceInfo;
        
        public static ConferenceInfo Load(String contentFile)
        {
            if (_conferenceInfo == null)
                _conferenceInfo = new ConferenceInfo();

            // Load basic attributes
            var reader = XElement.Load(contentFile);
            LoadBasicAttributes(reader);
            
            // Load Days
            LoadDays(reader);
            
            // Load Speakers
            LoadSpeakers(reader);

            // Load Sponsors
            LoadSponsors(reader);

            return _conferenceInfo;
        }

        private static void LoadBasicAttributes(XElement element)
        {
            var attribute = element.Attribute("Conference");
            if (attribute != null)
                _conferenceInfo.Conference = attribute.Value;

            attribute = element.Attribute("Description");
            if (attribute != null)
                _conferenceInfo.Description = attribute.Value;

            attribute = element.Attribute("Date");
            if (attribute != null)
                _conferenceInfo.Date = attribute.Value;

            attribute = element.Attribute("Location");
            if (attribute != null)
                _conferenceInfo.Location = attribute.Value;
        }

        private static void LoadDays(XElement agenderElement)
        {
            var qDays = from d in agenderElement.Elements("Days").Elements("Day") select d;
            var dayElements = qDays.ToList();

            foreach (var dayElement in dayElements)
            {
                var day = new Day();

                var attribute = dayElement.Attribute("Date");
                if (attribute != null)
                    day.Date = attribute.Value;
                attribute = dayElement.Attribute("Description");
                if (attribute != null)
                    day.Description = attribute.Value;

                // Timeslots for the day
                LoadTimeSlotsForDay(dayElement, day);

                _conferenceInfo.Days.Add(day);
            }
        }

        private static void LoadSpeakers(XElement agenderElement)
        {
            var qSpeakers = from s in agenderElement.Elements("Speakers").Elements("Speaker") select s;
            var speakerElements = qSpeakers.ToList();

            foreach (var speakerElement in speakerElements)
            {
                var speaker = new Speaker();

                // Speaker name/id (attributes)
                var attribute = speakerElement.Attribute("Name");
                if (attribute != null)
                    speaker.Name = attribute.Value;
                attribute = speakerElement.Attribute("Id");
                if (attribute != null)
                    speaker.Id = attribute.Value;

                // Speaker bio (child node)
                var bioElement = (from b in speakerElement.Elements("Bio") select b).FirstOrDefault();
                speaker.Bio = bioElement.Value;

                _conferenceInfo.Speakers.Add(speaker);
            }
        }

        private static void LoadSponsors(XElement agenderElement)
        {
            var qSponsors = from s in agenderElement.Elements("Sponsors").Elements("Sponsor") select s;
            var sponsorElements = qSponsors.ToList();

            foreach (var sponsorElement in sponsorElements)
            {
                var sponsor = new Sponsor();

                var attribute = sponsorElement.Attribute("Name");
                if (attribute != null)
                    sponsor.Name = attribute.Value;
                attribute = sponsorElement.Attribute("Picture");
                if (attribute != null)
                    sponsor.Picture = attribute.Value;
                attribute = sponsorElement.Attribute("Url");
                if (attribute != null)
                    sponsor.Url = attribute.Value;

                _conferenceInfo.Sponsors.Add(sponsor);
            }
        }

        private static void LoadTimeSlotsForDay(XElement dayElement, Day day)
        {
            var qTimeSlots = from ts in dayElement.Elements("TimeSlots").Elements("TimeSlot") select ts;
            var timeSlotElements = qTimeSlots.ToList();

            foreach (var timeSlotElement in timeSlotElements)
            {
                var slot = new TimeSlot();
                var attribute = timeSlotElement.Attribute("Time");
                if (attribute != null)
                    slot.StartTime = attribute.Value;
                attribute = timeSlotElement.Attribute("Description");
                if (attribute != null)
                    slot.Description = attribute.Value;

                // Load sessions for timeslot
                LoadSessionsForTimeSlot(timeSlotElement, slot);

                day.TimeSlots.Add(slot);
            }
        }

        private static void LoadSessionsForTimeSlot(XElement timeSlotElement, TimeSlot timeSlot)
        {
            var qSession = from s in timeSlotElement.Elements("Session") select s;
            var sessionElements = qSession.ToList();

            foreach (var sessionElement in sessionElements)
            {
                var session = new Session();

                var idElement = (from e in sessionElement.Elements("Id") select e).FirstOrDefault();
                session.Id = idElement.Value;
                var titleElement = (from e in sessionElement.Elements("Title") select e).FirstOrDefault();
                session.Title = titleElement.Value;
                var speakerElement = (from e in sessionElement.Elements("Speaker") select e).FirstOrDefault();
                session.Speaker = speakerElement.Value;
                var roomElement = (from e in sessionElement.Elements("Room") select e).FirstOrDefault();
                session.Room = roomElement.Value;
                var trackElement = (from e in sessionElement.Elements("Track") select e).FirstOrDefault();
                session.Track = trackElement.Value;
                var abstractElement = (from e in sessionElement.Elements("Abstract") select e).FirstOrDefault();
                session.Abstract = abstractElement.Value;

                timeSlot.Sessions.Add(session);
            }
        }
    }
}